// $Id: Filename.hh 9571 2009-04-13 19:37:05Z mthuurne $

#ifndef FILENAME_HH
#define FILENAME_HH

#include <string>

namespace openmsx {

class FileContext;
class CommandController;

/** This class represents a filename.
 * A filename is resolved in a certain context.
 * It is possible to query both the resolved and unresolved filename.
 *
 * Most file operations will want the resolved name, but for example
 * for savestates we (also) want the unresolved name.
 */
class Filename
{
public:
	Filename();
	explicit Filename(const std::string& filename);
	Filename(const std::string& filename, CommandController& controller);
	Filename(const std::string& filename, const FileContext& context);
	Filename(const std::string& filename, const FileContext& context,
	         CommandController& controller);

	const std::string& getOriginal() const;
	const std::string& getResolved() const;

	/** After a loadstate we prefer to use the exact same file as before
	  * savestate. But if that file is not available (possibly because
	  * snapshot is loaded on a different host machine), we fallback to
	  * the original filename.
	  */
	void updateAfterLoadState(CommandController& controller);

	/** Convenience method to test for empty filename.
	 * In any case getOriginal().empty() and getResolved().empty() return
	 * the same result. This method is a shortcut to either of these.
	 */
	bool empty() const;

	template<typename Archive>
	void serialize(Archive& ar, unsigned version);

private:
	// non-const because we want this class to be assignable
	// (to be able to store them in std::vector)
	std::string originalFilename;
	std::string resolvedFilename;
};

} // namespace openmsx

#endif
